@@ -1,8 +1,7 @@
 from .request_payload import AgentRequestPayload
-
-# from .response_payload import _AgentResponsePayload
+from .response_payload import AgentResponsePayload
 
 __all__ = [
     "AgentRequestPayload",
-    # "AgentResponsePayload",
+    "AgentResponsePayload",
 ]
