@@ -1,4 +1,6 @@
-from typing import Any, Union
+from typing import Any, Union, override
+
+from pydantic import Field
 
 from syncmaster_commons.abstract.baseclass import (
     SMBaseClass, ThirdPartyPayloadConsumedByAgent)
@@ -21,6 +23,13 @@ class AgentRequestPayload(SMBaseClass):
     """
     payload: Union[ThirdPartyPayloadConsumedByAgent,Any]
 
+    @property
+    def app_name(self) -> str:
+        """
+        Returns the name of the applicatio that the payload is associated with.
+        """
+        return self.payload.app_name
+
     @property
     def org_id(self) -> int:
         """
@@ -42,6 +51,13 @@ class AgentRequestPayload(SMBaseClass):
         """
         return self.payload.task_id
     
+    @property
+    def task_name(self) -> str:
+        """
+        Returns the task name.
+        """
+        return self.payload.task_name
+    
     @property
     def org_name(self) -> str:
         """
@@ -56,6 +72,34 @@ class AgentRequestPayload(SMBaseClass):
         """
         return self.payload.payload.get("messages", None)
     
+    @property
+    def streamer_payload(self) -> dict:
+        """
+        Returns the streamer payload.
+        """
+        return {
+            "messages": self.messages,
+            "user_id": self.user_id,
+            "org_id": self.org_id,
+            "org_name": self.org_name,
+            "org_id": self.org_id,
+            "task_id": self.task_id,
+            "task_name": self.task_name,
+            "incoming_msg_platform": self.app_name
+        }
+    
+    @override
+    def to_dict(self):
+        """
+        Provides a dictionary representation of the current instance, extracted from
+        the dictionary returned by the parent class.
+
+        Returns:
+            dict: The payload portion of the dictionary obtained from the parent class.
+        """
+        output_dict =  super().to_dict()
+        return output_dict["payload"]
+    
 
     @classmethod
     def from_dict(cls,request_payload: dict, client:str = None) -> "AgentRequestPayload":
@@ -67,7 +111,7 @@ class AgentRequestPayload(SMBaseClass):
             AgentRequestPayload: The AgentRequestPayload object created from the dictionary.
         """
         app_name = request_payload.get("app_name", None)
-        if client == "gupshup" or app_name == "gupshup":
+        if client == "WhatsApp" or app_name == "WhatsApp":
             payload = _AgentRequestPayloadGupshup.from_dict(request_payload) 
         else:
             raise ValueError(f"Client {client} is not supported.")
